# sb::Admin::EntryList
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::EntryList;
use strict;
use sb::Admin::List ();
use base qw( sb::Admin::List);
use vars qw( $VERSION );
$VERSION = '0.03';
# 0.03 [2009/03/19] removed br tags from the strings
# 0.02 [2007/11/05] added title_name
# 0.01 [2007/09/24] changed _generate_condition to handle searching body
# 0.00 [2007/09/09] generated
# --------------------------------------
# // defines
sub TEMPLATE    (){ 'list.html' };
sub NO_CATEGORY (){ '&nbsp;' };
sub LIST_COLUMN (){ 7 };
# --------------------------------------
# // override base class
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	return sb->string(' &gt; Articles') if ($self->mode eq 'list');
	return;
}
# --------------------------------------
# // public functions - callback
sub callback
{
	my $self = shift;
	$self->{'cat'} = { sb::Data->load_as_hash('Category') };
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = '';
	my @sels = split("\0",$self->cgi->value('sel'));
	my @ents = sb::Data->load('Entry','cond'=>{'id'=>\@sels},'detail'=>'on');
	ACTION_SWITCH: {
		$_ = $self->cgi->value('regi_action');
		/^del$|^del_htm$/ && do
		{ # delete selected articles
			my $del_htm = ($_ eq 'del_htm');
			foreach my $ent (@ents)
			{
				my $file = $ent->file_path($self->{'cat'});
				unlink($file) if ($del_htm and -e $file);
				my @coms = sb::Data->load('Message','cond'=>{'eid'=>$ent->id});
				if (@coms)
				{
					map { $_->erase() } @coms;
					sb::Data->update(@coms);
				}
				my @tbs = sb::Data->load('Trackback','cond'=>{'eid'=>$ent->id});
				if (@tbs)
				{
					map { $_->erase() } @tbs;
					sb::Data->update(@tbs);
				}
				$ent->erase();
			}
			$msg = sprintf(sb->string('Deleted %d item(s).'),$#ents + 1) . "\n";
			last ACTION_SWITCH;
		};
		/^(acm|atb|stat)(\d)$/ && do
		{ # change the status of selected articles
			my $targ = $1;
			my $new = $2;
			foreach my $ent (@ents)
			{
				if ($targ eq 'stat')
				{
					my $cat = ($ent->cat ne '') ? $self->{'cat'}->{$ent->cat} : undef;
					$new = 2 if ($new == 1 and $cat and $cat->get_option('top'));
				}
				$ent->$targ($new);
			}
			$msg = sb->string('Modified.') . "\n";
			last ACTION_SWITCH;
		};
		/^none$|^\d+$/ && do
		{ # change the category of selected articles
			my $new = ($_ eq 'none') ? -1 : $self->{'cat'}->{$_};
			foreach my $ent (@ents)
			{
				$ent->cat(($new) ? $new->id : -1);
				$ent->stat(($new and $new->get_option('top')) ? 2 : 1) if ($ent->stat > 0);
			}
			$msg = sb->string('Modified.') . "\n";
			last ACTION_SWITCH;
		};
	} # end of ACTION_SWITCH
	sb::Data->update(@ents) if (@ents);
	$msg .= sb->string('You need to rebuild your weblog to reflect the changes completely.');
	$msg .= sprintf(sb->string('&#8658; <a href="%s">Rebuild site</a>'),$self->script_path . '?__mode=build') . "\n";
	return $self->_display('message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		'setup'   => $self->setup_list(),
		@_
	);
	my @cats = sort { $b->order <=> $a->order } values(%{$self->{'cat'}});
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	my $dispsort = ( $self->cgi->value('dispsort') ne '' ) ? $self->cgi->value('dispsort') : 'date';
	my $page = $self->get_current_page();
	my @entry = sb::Data->load('Entry',
		'sort'  => $dispsort,
		'order' => 1,
		'num'   => $param{'setup'}->{'dispnum'},
		'bgn'   => $page * $param{'setup'}->{'dispnum'},
		'cond'  => $self->_generate_condition(),
	);
	$temp->num(0);
	$temp->tag('sb_list_page'=>$self->display_pagelink(
			'mode'    => 'list',
			'column'  => LIST_COLUMN,
			'all'     => sb::Data->matched,
			'printed' => $#entry + 1,
			'num'     => $param{'setup'}->{'dispnum'},
			'params'  => ['dispsort','disptype','dispword','dispdate','dispnum','dispcat'],
		)
	);
	$temp->tag('sb_entry_changecat'=>
		'<option value="none">' . sb->string('No category') . '</option>' .
		$self->category_selector('cat'=>\@cats)
	);
	$temp->tag('sb_entry_dispcat'=>
		$self->category_selector('cat'=>\@cats,'now'=>$self->cgi->value('dispcat'))
	);
	$temp->tag('sb_entry_selectcat'=>
		$self->category_selector('cat'=>\@cats,'val'=>'cat%d_')
	);
	$self->dispnum_selector(
		'cms' => $temp,
		'now' => $param{'setup'}->{'dispnum'},
	);
	$self->monthly_selector(
		'cms'  => $temp,
		'tag'  => 'sb_entry_dispdate',
		'data' => 'Entry',
	);
	foreach my $key ('dispsort','disptype')
	{
		$self->select_option(
			'cms'      => $temp,
			'tag'      => 'sb_' . $key . '_',
			'selected' => ($key eq 'dispsort') ? $dispsort : $self->cgi->value($key),
		);
	}
	$temp->block('sb_entry_delcheck'=>1) if ($self->conf->value('conf_entry_archive') eq 'Individual');
	$self->listmain(
		'template' => $temp,
		'block'    => 'sb_entry_list',
		'objects'  => \@entry,
		'tags'     => {
			'sb_entry_id'        => 'id',
			'sb_entry_title'     => sub { (shift)->display_entry_title(@_) },
			'sb_entry_author'    => \&_display_author,
			'sb_entry_category'  => \&_display_category,
			'sb_entry_date'      => 'date',
			'sb_entry_status'    => sub { (shift)->display_entry_status(@_) },
			'sb_entry_catclass'  => 'cat',
			'sb_entry_statclass' => sub { (shift)->get_entry_status(@_) },
			'sb_entry_sel'       => sub { (shift)->display_checkbox(@_) },
		},
	);
	if ($self->cgi->value('disptype') ne '')
	{
		$temp->num(0);
		$temp->tag('sb_dispword'=>sb::Text->new($self->cgi->value('dispword'))->entitized());
	}
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - display
sub _generate_condition
{
	my $self = shift;
	my %cond = ();
	if ($self->cgi->value('dispword') ne '')
	{
		my $word = $self->cgi->value('dispword');
		SWITCH_CHECK_COND:
		{
			$_ = $self->cgi->value('disptype');
			/^subj/ && do
			{
				$cond{'subj'} = '/' . $word . '/';
				last SWITCH_CHECK_COND;
			};
			/^body/ && do
			{
				$cond{'body'} = '/' . $word . '/';
				$cond{'more'} = '/' . $word . '/';
				$cond{'sum'}  = '/' . $word . '/';
				last SWITCH_CHECK_COND;
			};
			/^auth/ && do
			{
				$cond{'auth'} = [];
				my @users = sb::Data->load('User','cond'=>{'real'=>'/' . $word . '/','name'=>'/' . $word . '/'});
				foreach my $user (@users)
				{
					push(@{$cond{'auth'}},$user->id);
				}
				last SWITCH_CHECK_COND;
			};
			/^cat/ && do
			{
				$cond{'cat'} = [];
				my @cats = sb::Data->load('Category','cond'=>{'name'=>'/' . $word . '/'});
				foreach my $cat (@cats)
				{
					push(@{$cond{'cat'}},$cat->id);
				}
				last SWITCH_CHECK_COND;
			};
		}
	}
	if ($self->cgi->value('dispcat') ne '')
	{
		$cond{'cat'} = $self->cgi->value('dispcat');
		$cond{'__combo'} = { 'cat' => 'add' , 'add' => ',' . $self->cgi->value('dispcat') . ',' };
	}
	if ($self->cgi->value('dispdate') ne '')
	{
		$cond{'date'} = sb::Utils->create_date_condition($self->cgi->value('dispdate'));
		$cond{'__range'} = { 'date' => 'tz' };
	}
	return \%cond;
}
# --------------------------------------
# // private functions - list handlers
sub _display_author
{
	my $self = shift;
	my $obj  = shift;
	my $pid  = ( defined($self->users->{$obj->auth}) ) ? $obj->auth : 0;
	return $self->users->{$pid}->real;
}
sub _display_category
{
	my $self = shift;
	my $obj  = shift;
	my $cat  = ($obj->cat == -1) ? undef : $self->{'cat'}->{$obj->cat};
	return ($cat) ? $cat->fullname($self->{'cat'}) : NO_CATEGORY;
}
1;
__END__
