# sb::Admin::Link
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Link;
use strict;
use sb::Admin::List ();
use base qw( sb::Admin::List );
use vars qw( $VERSION );
$VERSION = '0.07';
# 0.07 [2009/03/19] removed br tags from the strings
# 0.06 [2009/02/09] modified title_name to make it clearer
# 0.05 [2007/11/05] added title_name
# 0.04 [2007/09/16] ported from Serene Bach 2.*
# 0.03 [2005/07/20] changed _update_link to change group correctly, changed _is_editable to enter group screen
# 0.02 [2005/07/09] fixed a bug to display description of a link correctly
# 0.01 [2005/06/29] now, links can be grouped
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub TEMPLATE        (){ 'link.html' };
sub LIST_COLUMN     (){ 7 };
sub DENIED_CHECK    (){ '-' };
sub DEFAULT_COLUMN  (){ '-' };
sub DEFAULT_TARGET  (){ '' }
sub LIST_GROUP_FORM (){ '<strong>%s</strong> (%d)' };
sub LIST_SITE_FORM  (){ '<a href="%s">%s</a>' };
# --------------------------------------
# // override base class
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	return sb->string(' &gt; Links');
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	$self->{'group'} = {};
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = undef;
	if ($self->regi() eq 'site' or $self->regi() eq 'group')
	{ # edit or create a link
		$msg = $self->_update_link_details()
	}
	elsif ($self->cgi->value('submit') ne '')
	{ # edit or delete selected link(s)
		$msg = $self->_update_linklist();
	}
	else
	{ # change the order of links
		$msg = $self->_update_order();
	}
	$self->build_list('link_list');
	return $self->_display('message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		'setup'   => $self->setup_list(),
		@_
	);
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	my @all = sb::Data->load('Link','sort'=>'order','order'=>1);
	my @links = ();
	my @group = ();
	my $target = undef;
	$target = sb::Data->load('Link','id'=>$self->cgi->value('lid')) if ($self->cgi->value('lid') ne '');
	foreach my $lnk (@all)
	{
		push(@group,$lnk) if ($lnk->is_group);
		$self->{'group'}->{$lnk->type}++ if ($lnk->type =~ /^\d+$/);
	}
	if (!$target)
	{
		foreach my $lnk (@all)
		{
			push(@links,$lnk) if ($lnk->type !~ /^\d+$/);
		}
	}
	elsif ($target->is_group)
	{
		foreach my $lnk (@all)
		{
			push(@links,$lnk) if ($lnk->type eq $target->id);
		}
	}
	if (!$target or $target->is_group)
	{ # display list table
		my $matched = @links;
		my $page = $self->get_current_page();
		@links = splice(@links,$page * $param{'setup'}->{'dispnum'},$param{'setup'}->{'dispnum'});
		$temp->num(0);
		$temp->tag('page_now'=>$page);
		$temp->tag('sb_list_page'=>$self->display_pagelink(
				'mode'    => 'link',
				'column'  => LIST_COLUMN,
				'all'     => $matched,
				'printed' => $#links + 1,
				'num'     => $param{'setup'}->{'dispnum'},
				'params'  => ['dispnum'],
			)
		);
		$self->dispnum_selector(
			'cms'  => $temp,
			'now'  => $param{'setup'}->{'dispnum'},
		);
		$self->listmain(
			'template' => $temp,
			'block'    => 'sb_link_list',
			'objects'  => \@links,
			'tags'     => {
				'sb_link_id'        => 'id',
				'sb_link_name'      => \&_display_name,
				'sb_link_edit'      => \&_is_editable,
				'sb_link_dispstat'  => \&_display_status,
				'sb_link_disp'      => 'disp',
				'sb_link_sel'       => \&_display_checkbox,
			},
		);
		$self->order_marker(
			'template' => $temp,
			'objects'  => \@links,
			'matched'  => $matched,
			'tag'      => 'sb_link_order',
			'page'     => $page,
			'number'   => $param{'setup'}->{'dispnum'},
		);
		if ($target)
		{ # -- group mode
			$temp->num(0);
			$temp->tag('sb_link_groupname'=>$target->name);
			$temp->tag('sb_link_grouptext'=>$target->text);
			$temp->tag('sb_link_groupid'=>$target->id);
			$temp->tag('sb_link_navi'=>$target->name);
			$temp->tag('sb_link_typesel'=>$self->_group_selector('group'=>\@group,'now'=>$target->id));
			$temp->block('sb_link_oldgroup_title' => 1);
			if ($self->has_permission('id'=>$target->user))
			{
				$temp->block('sb_link_oldgroup_button' => 1);
				$temp->block('sb_link_group_edit'      => 1);
			}
		}
		else
		{
			$temp->num(0);
			$temp->tag('sb_link_typesel'=>$self->_group_selector('group'=>\@group));
			$temp->block('sb_link_newgroup_title'  => 1);
			$temp->block('sb_link_newgroup_button' => 1);
			$temp->block('sb_link_group_edit'      => 1);
		}
		$temp->num(0);
		$temp->tag('sb_link_typechange'=>$self->_group_selector('group'=>\@group,'label'=>'group'));
		$temp->tag('sb_link_sitetarget'=>DEFAULT_TARGET);
		$temp->block('sb_link_newsite_title'  => 1);
		$temp->block('sb_link_newsite_button' => 1);
		$temp->block('sb_link_group'          => 1);
	}
	else
	{ # -- individual mode
		my $member = undef;
		my $navigation = '';
		if ($target->type =~ /^\d+$/)
		{
			foreach my $grp (@group)
			{
				if ($grp->id eq $target->type)
				{
					$navigation .= '<a href="' . $self->get_script_path . '?__mode=link&amp;';
					$navigation .= 'lid=' . $grp->id . '">' . $grp->name . '</a> &gt; ';
				}
			}
		}
		$temp->num(0);
		$temp->tag('sb_link_navi'       => $navigation . $target->name);
		$temp->tag('sb_link_sitename'   => $target->name);
		$temp->tag('sb_link_siteurl'    => $target->entitize('url'));
		$temp->tag('sb_link_sitetext'   => $target->text);
		$temp->tag('sb_link_sitetarget' => $target->target);
		$temp->tag('sb_link_typesel'    => $self->_group_selector('group'=>\@group,'now'=>$target->type));
		$temp->tag('sb_link_siteid'     => $target->id);
		$temp->block('sb_link_oldsite_title'  => 1);
		$temp->block('sb_link_oldsite_button' => 1);
	}
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - update
sub _update_link_details
{
	my $self = shift;
	my $new = undef;
	my %elem = ();
	my $target = undef;
	$target = sb::Data->load('Link','id'=>$self->cgi->value('lid')) if ($self->cgi->value('lid') ne '');
	if ($self->regi() eq 'site')
	{
		%elem = (
			'name'   => sb::Text->new($self->cgi->value('link_name'))->entitized(),
			'url'    => $self->cgi->value('link_url'),
			'text'   => sb::Text->new($self->cgi->value('link_text'))->entitized(),
			'target' => sb::Text->new($self->cgi->value('link_target'))->entitized(),
			'user'   => ( $self->user ) ? $self->user->id : 0,
			'type'   => $self->cgi->value('link_type'),
		);
	}
	else
	{
		%elem = (
			'name'   => sb::Text->new($self->cgi->value('link_group_name'))->entitized(),
			'url'    => '',
			'text'   => sb::Text->new($self->cgi->value('link_group_text'))->entitized(),
			'target' => '',
			'user'   => ( $self->{'user'} ) ? $self->{'user'}->id : 0,
			'type'   => '',
		);
	}
	if ($target)
	{ # modified existed link or group
		foreach my $key (keys(%elem))
		{
			$target->$key($elem{$key});
		}
	}
	else
	{ # added new link or group
		$target = sb::Data->add('Link',%elem);
		$new = 1;
	}
	if ($target)
	{
		$target->set_as_group if ($self->regi() eq 'group');
		sb::Data->update($target);
		return ($new)
			? sb->string('Your request has been processed.')
			: sb->string('Modified.') . "\n";
	}
	return sb->string('Unknown error.');
}
sub _update_linklist
{
	my $self = shift;
	my @sels = split("\0",$self->cgi->value('sel'));
	my @links = sb::Data->load('Link','cond'=>{'id'=>\@sels});
	my @del_group = ();
	ACTION_SWITCH: {
		$_ = $self->cgi->value('regi_action');
		/^del$/ && do
		{ # delete
			foreach my $lnk (@links)
			{
				push(@del_group,$lnk->id) if ($lnk->is_group);
				$lnk->erase;
			}
			last ACTION_SWITCH;
		};
		/^disp(\d)$/ && do
		{ # change the display status
			my $new = $1;
			foreach my $lnk (@links)
			{
				$lnk->disp($new);
			}
			last ACTION_SWITCH;
		};
		/^group(\d*)$/ && do
		{ # change the group
			my $new = $1;
			foreach my $lnk (@links)
			{
				next if ($lnk->is_group);
				$lnk->type($new);
			}
			last ACTION_SWITCH;
		};
	} # end of ACTION_SWITCH
	sb::Data->update(@links) if (@links);
	if (@del_group)
	{
		my @change = sb::Data->load('Link','cond'=>{'type'=>\@del_group});
		foreach my $lnk (@change)
		{
			$lnk->type('');
		}
		sb::Data->update(@change) if (@change);
	}
	return ($self->cgi->value('regi_action') eq 'del')
		? sprintf(sb->string('Deleted %d item(s).'),$#links + 1) . "\n"
		: sb->string('Modified.') . "\n";
}
sub _update_order
{
	my $self = shift;
	my @ids = split("\0",$self->cgi->value('sel_id'));
	my @links = sb::Data->load('Link','cond'=>{'id'=>\@ids});
	my @buf = sb::Data->load('Link','sort'=>'order','order'=>1); # load all links
	my @all = ();
	my $target = undef;
	$target = sb::Data->load('Link','id'=>$self->cgi->value('lid')) if ($self->cgi->value('lid') ne '');
	if (!$target)
	{ # id is not indicated.
		foreach my $lnk (@buf)
		{
			push(@all,$lnk) if ($lnk->type !~ /^\d+$/);
		}
	}
	elsif ($target->is_group)
	{ # in particular group
		foreach my $lnk (@buf)
		{
			push(@all,$lnk) if ($lnk->type eq $target->id);
		}
	}
	else
	{ # it does not happen normally
		return sb->string('Unknown error.');
	}
	my $order = undef;
	foreach my $lnk (@links)
	{
		next if ($self->cgi->value('dn' . $lnk->id) eq '' and $self->cgi->value('up' . $lnk->id) eq '');
		$order = $lnk;
		last;
	}
	if ($order)
	{
		@links = $self->change_order(
			'data'      => \@all,
			'target'    => $order,
			'direction' => ($self->cgi->value('up' . $order->id) ne '') ? +1 : -1,
		);
	}
	sb::Data->update(@links) if (@links);
	return sb->string('Modified.') . "\n";
}
# --------------------------------------
# // private functions - display
sub _group_selector
{
	my $self = shift;
	my %param = (
		'group' => undef,
		'now'   => undef,
		'label' => '',
		@_
	);
	return if (!$param{'group'});
	my $selector = '';
	foreach my $group ( @{$param{'group'}} )
	{
		$selector .= '<option value="' . $param{'label'} . $group->id . '"';
		$selector .= ' selected="selected"' if ($param{'now'} eq $group->id);
		$selector .= '>' . $group->name . '</option>' . "\n";
	}
	return $selector;
}
# --------------------------------------
# // private functions - list handlers
sub _is_editable
{
	my $self = shift;
	my $obj  = shift;
	my $url  = $self->get_script_path . '?__mode=link&amp;lid=' . $obj->id;
	return ( $self->has_permission('id'=>$obj->user) or $obj->is_group ) 
		? '<a href="' . $url . '">' . sb->string('Edit') . '</a>' 
		: DENIED_CHECK;
}
sub _display_name
{
	my $self = shift;
	my $obj  = shift;
	return 
		  ($obj->is_group) ? sprintf(LIST_GROUP_FORM,$obj->name,int($self->{'group'}->{$obj->id}))
		: ($obj->url)      ? sprintf(LIST_SITE_FORM,$obj->url,$obj->name)
		:                    $obj->name;
}
sub _display_status
{
	my $self = shift;
	my $obj  = shift;
	return $self->list_status(
		'stat'   => $obj->disp,
		'string' => sb->string('open:close'),
	);
}
sub _display_checkbox
{
	my $self = shift;
	my $obj  = shift;
	return ( $self->has_permission('id'=>$obj->user) )
		? '<input type="checkbox" name="sel" value="' . $obj->id . '" />'
		: DENIED_CHECK;
}
1;
__END__
